import { toCamelCase } from 'web-utility';
import { ProxyElement } from './Proxy';
import { EventKeyPattern } from './utility';
export class ECOptionElement extends ProxyElement {
    get chartTagName() {
        return toCamelCase(this.tagName.replace(/^ec-/i, '').toLowerCase());
    }
    get isSeries() {
        return this.chartTagName.endsWith('Chart');
    }
    get chartName() {
        return this.isSeries ? this.chartTagName.replace(/Chart$/, '') : null;
    }
    get eventSelector() {
        return [this.chartName || this.chartTagName, this['type']]
            .filter(Boolean)
            .join('.');
    }
    connectedCallback() {
        for (const [key, value] of Object.entries(this.toJSON()))
            if (EventKeyPattern.test(key) && typeof value === 'function')
                this.listen(key.slice(2), value);
        this.update();
    }
    setProperty(key, value) {
        super.setProperty(key, value);
        if (this.isConnected)
            this.update();
    }
    update() {
        const data = this.toJSON();
        this.dispatchEvent(new CustomEvent('optionchange', {
            bubbles: true,
            detail: this.isSeries
                ? { series: [Object.assign(Object.assign({}, data), { type: this.chartName })] }
                : { [this.chartTagName]: data }
        }));
    }
    listen(event, handler) {
        var _a;
        if (this.isConnected)
            (_a = this.closest('ec-chart')) === null || _a === void 0 ? void 0 : _a.onChild(event, this.eventSelector, handler);
    }
    forget(event, handler) {
        var _a;
        if (this.isConnected)
            (_a = this.closest('ec-chart')) === null || _a === void 0 ? void 0 : _a.offChild(event, this.eventSelector, handler);
    }
    setAttribute(name, value) {
        super.setAttribute(name, value);
        const key = toCamelCase(name);
        if (key in Object.getPrototypeOf(this))
            return;
        this[key] = name === value || value;
    }
}
//# sourceMappingURL=Option.js.map