/// <reference types="lodash" />
import { EChartsOption, ResizeOpts } from 'echarts';
import { init } from 'echarts/core';
import { ECBasicOption } from 'echarts/types/dist/shared';
import { CustomElement } from 'web-utility';
import { ProxyElement } from '../Proxy';
import { ZRElementEventHandler, ZRElementEventName } from '../utility';
export type EChartsElementEventHandler = Partial<Record<`on${Capitalize<ZRElementEventName>}`, ZRElementEventHandler>>;
export interface EChartsElementProps extends ECBasicOption, EChartsElementEventHandler {
    theme?: Parameters<typeof init>[1];
    initOptions?: Omit<Parameters<typeof init>[2], 'renderer'>;
    resizeOptions?: ResizeOpts;
}
export type EChartsElementState = EChartsElementProps & EChartsOption;
export declare abstract class EChartsElement extends ProxyElement<EChartsElementState> implements CustomElement {
    #private;
    get renderer(): string;
    get options(): ECBasicOption;
    constructor();
    connectedCallback(): void;
    disconnectedCallback(): void;
    setOption(data: EChartsOption): Promise<void>;
    setProperty(key: string, value: any): void;
    listen(event: ZRElementEventName, handler: ZRElementEventHandler): void;
    onChild(event: ZRElementEventName, selector: string, handler: ZRElementEventHandler): void;
    forget(event: ZRElementEventName, handler: ZRElementEventHandler): void;
    offChild(event: ZRElementEventName, selector: string, handler: ZRElementEventHandler): void;
    handleResize: import("lodash").DebouncedFunc<() => void>;
}
